<?php

namespace App\Filament\Admin\Resources;

use App\Enums\IconEnum;
use App\Models\Customer;
use App\Models\Invoice;
use Filament\Forms\Components\DatePicker;
use Filament\Forms\Form;
use Filament\Resources\Resource;
use Filament\Support\Enums\FontWeight;
use Filament\Tables;
use Filament\Tables\Columns\Summarizers\Summarizer;
use Filament\Tables\Table;
use Illuminate\Database\Eloquent\Model;

class CustomerReportResource extends Resource
{
    protected static ?string $model = Customer::class;

    protected static ?string $navigationIcon = IconEnum::CUSTOMER_SALES->value;

    protected static ?string $navigationGroup = 'Reports';

    protected static ?string $navigationLabel = 'Customer Reports';

    protected static ?int $navigationSort = 4;

    public static function form(Form $form): Form
    {
        return $form
            ->schema([
                //
            ]);
    }

    public static function table(Table $table): Table
    {
        return $table
            ->columns([
                Tables\Columns\TextColumn::make('company_name')
                    ->label('Customer')
                    ->sortable()
                    ->searchable()
                    ->extraHeaderAttributes(['class' => 'w-full']),

                Tables\Columns\TextColumn::make('subtotal')
                    ->money()
//                    ->summarize(Summarizer::make()->using(function ($query, Table $table) {
//                        $createdAt    = $table->getfilter('created_at')->getstate()['created_at'] ?? '1900-01-01';
//                        $createdUntil = $table->getfilter('created_until')->getstate()['created_until'] ?? '2100-01-01';
//
//                        $invoiceSum = invoice::wherebetween('date', [$createdAt, $createdUntil])->sum('subtotal');
//
//                        return '$'.number_format(round($invoiceSum, 2), 2, '.', ',');
//                    }))
                    ->alignRight()
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getSubtotalAttribute(
                            $table->getFilter('created_at')->getState()['created_at'],
                            $table->getFilter('created_until')->getState()['created_until']
                        );
                    }),

                Tables\Columns\TextColumn::make('gst')
                    ->label('GST')
                    ->money()
                    ->alignRight()
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getGstAttribute(
                            $table->getFilter('created_at')->getState()['created_at'],
                            $table->getFilter('created_until')->getState()['created_until']
                        );
                    }),

                Tables\Columns\TextColumn::make('pst')
                    ->label('PST')
                    ->money()
                    ->alignRight()
                    ->getStateUsing(function (Table $table, Customer $record) {
                        return $record->getPstAttribute(
                            $table->getFilter('created_at')->getState()['created_at'],
                            $table->getFilter('created_until')->getState()['created_until']
                        );
                    }),

                Tables\Columns\TextColumn::make('total')
                    ->money()
//                    ->summarize(summarizer::make()->using(function ($query, table $table) {
//                        $createdAt    = $table->getfilter('created_at')->getstate()['created_at'] ?? '1900-01-01';
//                        $createdUntil = $table->getfilter('created_until')->getstate()['created_until'] ?? '2100-01-01';
//
//                        $invoiceSum = invoice::wherebetween('date', [$createdAt, $createdUntil])->sum('total');
//
//                        return '$'.number_format(round($invoiceSum, 2), 2, '.', ',');
//                    }))
                    ->weight(FontWeight::Bold)
                    ->alignRight()
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getTotalAttribute(
                            $table->getFilter('created_at')->getState()['created_at'],
                            $table->getFilter('created_until')->getState()['created_until']
                        );
                    }),
            ])
            ->filters([
                Tables\Filters\Filter::make('created_at')
                    ->form([
                        DatePicker::make('created_at')
                            ->label('From date'),
                    ]),

                Tables\Filters\Filter::make('created_until')
                    ->form([
                        DatePicker::make('created_until')
                            ->label('Until date'),
                    ]),
            ])
            ->actions([
            ])
            ->bulkActions([
            ]);
    }

    public static function canAccess(): bool
    {
        return auth()->user()->is_admin ?? false;
    }

    public static function getRelations(): array
    {
        return [
            //
        ];
    }

    public static function getPages(): array
    {
        return [
            'index' => \App\Filament\Admin\Resources\CustomerReportResource\Pages\ListCustomerReports::route('/'),
        ];
    }
}
